﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web;
using System.Web.UI;
using Dynamicweb.Controls;
using Dynamicweb.Controls.Extensibility;
using Dynamicweb.Ecommerce.Orders;
using Dynamicweb.Extensibility;
using Dynamicweb.Extensibility.AddIns;

namespace Dynamicweb.Ecommerce.LiveIntegration.AddIns
{
    [AddInName("Resend order")]
    [AddInDescription("Resend order from backend")]
    [AddInTarget(RibbonBarAddInTarget.eCom.OrderEdit)]
    [AddInTarget(RibbonBarAddInTarget.eCom.OrderList)]
    public class ResendOrderAddIn : RibbonBarAddIn
    {
        private string SessionKey
        {
            get
            {
                string key = "LiveIntegration.TransferredOrder";
                Order order = base.Ribbon.DataContext.DataSource as Order;
                if (order != null)
                {
                    key += order.Id;
                }
                return key;
            }
        }

        private readonly string TransferOrdersButtonClickedSessionKey = "TransferOrdersButton_Click";

        public ResendOrderAddIn(RibbonBar ribbon)
            : base(ribbon)
        {
        }

        public override void Load()
        {
            // The code below is just a sample implementation that shows how to access the Ecommerce order associated with this RibbonBarAddIn.
            // To learn more about RibbonBarAddIns, check out: http://developer.dynamicweb-cms.com/documentation/for-developers/cms-extensibility/ribbon-bar.aspx

            //If Order list is loaded
            if (base.Ribbon.DataContext.DataSource is Order[])
            {
                RibbonBarButton button = CreateTransferOrderButton();

                Order[] orders = base.Ribbon.DataContext.DataSource as Order[];
                if (orders.Any(o => o != null))
                {
                    button.OnClientClick = "if(!confirm('Transfer selected orders to NAV?')) { return false; };";
                    button.EnableServerClick = true;
                    button.Click += TransferOrdersButton_Click;
                }
                else
                {
                    button.OnClientClick = "alert('No orders to transfer');";
                }
            }
            else
            {
                //If Order edit is loaded
                if (base.Ribbon.DataContext.DataSource is Order)
                {
                    // Access the DataContext which is an Order for this AddIn.
                    Order order = base.Ribbon.DataContext.DataSource as Order;
                    if (order != null)
                    {
                        RibbonBarButton button = CreateTransferOrderButton();
                        if (string.IsNullOrEmpty(order.IntegrationOrderId))
                        {
                            button.OnClientClick = "if(!confirm('Transfer " + order.Id + " to NAV?')) { return false; };";
                            button.EnableServerClick = true;
                            button.Click += TransferOrderButton_Click;
                        }
                        else
                        {
                            button.OnClientClick = "alert('Order has been transferred');";
                        }
                    }
                }
            }

            //release flag for the orders export button click
            HttpContext.Current.Session[TransferOrdersButtonClickedSessionKey] = null;
        }

        public override void Render(HtmlTextWriter writer)
        {
            if (HttpContext.Current.Session[TransferOrdersButtonClickedSessionKey] != null)
            {
                List<Order> orders = GetSelectedOrders();
                if(orders != null)
                {
                    HttpContext.Current.Session[SessionKey] = TransferOrders(orders);
                }
            }            
            if (HttpContext.Current.Session[SessionKey] != null && !string.IsNullOrEmpty(HttpContext.Current.Session[SessionKey].ToString()))
            {
                writer.Write("<script type='text/javascript'>alert('" + HttpUtility.HtmlEncode(HttpContext.Current.Session[SessionKey]) + "')</script>");
                HttpContext.Current.Session.Remove(SessionKey);
            }            
            base.Render(writer);
        }

        /// <summary>
        /// Occures when the button was clicked from edit order page
        /// </summary>        
        private void TransferOrderButton_Click(object sender, EventArgs e)
        {
            string output = string.Empty;

            Order order = base.Ribbon.DataContext.DataSource as Order;
            if (order != null)
            {
                if (string.IsNullOrEmpty(order.IntegrationOrderId))
                {
                    if (TransferOrder(order))
                    {
                        output = "Order successfully transferred to NAV";
                    }
                    else
                    {
                        output = "Error creating order in NAV. See LiveIntegration log for details";
                    }
                }
                else
                {
                    output = "Order already transferred";
                }
            }

            HttpContext.Current.Session[SessionKey] = output;
        }

        /// <summary>
        /// Occures when the button was clicked from orders list page
        /// </summary>        
        private void TransferOrdersButton_Click(object sender, EventArgs e)
        {            
            //flag that the button was clicked - need to execute orders export in Render step(as the selected orderslist is still not available here)
            HttpContext.Current.Session[TransferOrdersButtonClickedSessionKey] = TransferOrdersButtonClickedSessionKey;
        }

        private RibbonBarButton CreateTransferOrderButton()
        {
            RibbonBarGroup group = base.CreateDefaultContainer();
            RibbonBarButton button = new RibbonBarButton();
            group.AddItem(button);

            button.Text = "Transfer to NAV";
            button.Image = Dynamicweb.Controls.Icons.Icon.Type.Export;
            // Alternatively, specify your own Image path:
            // button.ImagePath = "/Admin/SomeImage.png";

            button.Size = Dynamicweb.Controls.Icons.Icon.Size.Large;

            return button;
        }

        private bool TransferOrder(Order order)
        {
            OrderHandler.UpdateOrder(order);
            return !string.IsNullOrEmpty(order.IntegrationOrderId);
        }
        
        private string TransferOrders(List<Order> orders)
        {
            string output = string.Empty;            
            if (orders != null)
            {
                if (orders.Count > 0)
                {
                    List<string> exportedOrders = new List<string>();
                    List<string> alreadyExportedOrders = new List<string>();

                    foreach (Order o in orders)
                    {
                        if (string.IsNullOrEmpty(o.IntegrationOrderId))
                        {
                            if (TransferOrder(o))
                            {
                                exportedOrders.Add(o.Id);
                            }
                        }
                        else
                        {
                            alreadyExportedOrders.Add(o.Id);
                        }
                    }
                    if (alreadyExportedOrders.Count > 0 && alreadyExportedOrders.Count == orders.Count())
                    {
                        output = "All selected orders are already transferred to NAV";
                    }
                    else if (exportedOrders.Count > 0 || alreadyExportedOrders.Count > 0)
                    {
                        if ((exportedOrders.Count + alreadyExportedOrders.Count) == orders.Count())
                        {
                            output = "All selected orders were successfully transferred to NAV";
                        }
                        else
                        {
                            if (alreadyExportedOrders.Count > 0)
                            {
                                output += string.Format("Orders with IDs [{0}] are already transferred to NAV. ", string.Join(",", alreadyExportedOrders));
                            }
                            if (exportedOrders.Count > 0)
                            {
                                output += string.Format("Orders with IDs [{0}] were successfully transferred to NAV. ", string.Join(",", exportedOrders));
                            }
                            output += string.Format("Orders with IDs [{0}] were not transferred to NAV. See LiveIntegration log for details",
                                string.Join(",", orders.Where(o => !(exportedOrders.Contains(o.Id) || alreadyExportedOrders.Contains(o.Id))).Select(o => o.Id).Distinct().ToArray()));
                        }
                    }
                    else
                    {
                        output = "All selected orders were not transferred to NAV. See LiveIntegration log for details";
                    }
                }
                else
                {
                    output = "No orders selected for transfer";
                }
            }
            else
            {
                output = "Can not load selected orders from the order list";
            }
            return output;
        }        

        private List<Order> GetSelectedOrders()
        {
            List<Order> orders = new List<Order>();
            if (HttpContext.Current.Session["DW.Controls.Tree.List"] != null && HttpContext.Current.Session["DW.Controls.Tree.List"] is Dynamicweb.Controls.ListRowCollection)
            {
                //Get selected orders from order list
                ListRowCollection rows = HttpContext.Current.Session["DW.Controls.Tree.List"] as ListRowCollection;
                if (rows != null && rows.Count > 0 && rows.Any(r => r.Selected))
                {
                    foreach (string id in rows.Where(r => r.Selected).Select(r => r.ItemID).Distinct())
                    {
                        Order o = Order.GetOrderById(id);
                        if (o != null)
                        {
                            orders.Add(o);
                        }
                    }
                }
            }
            else
            {
                //at some events the list rows collection can not be retrieved
                orders = null;
            }
            return orders;
        }
    }
}
