﻿using System;
using System.IO;
using System.Web;
using System.Collections;
using System.Collections.Generic;
using System.Xml.Linq;
using Dynamicweb.Extensibility;
using Dynamicweb.Extensibility.Editors;
using Dynamicweb.Rendering;
using Dynamicweb.Ecommerce.LiveIntegration.Configuration;
using System.Diagnostics;
using Dynamicweb.Extensibility.AddIns;
using Dynamicweb.Ecommerce.Integration;
using Dynamicweb.DataIntegration.Integration.ERPIntegration;
using Dynamicweb.Security.UserManagement;
using Dynamicweb.Environment;
using Dynamicweb.Ecommerce.International;
using System.Linq;

namespace Dynamicweb.Ecommerce.LiveIntegration.Admin
{
    [AddInName(Constants.AddInName), AddInLabel("ERP Live Integration"), AddInIgnore(false)]
    public class LiveIntegrationAddIn : BaseLiveIntegrationAddIn, IDropDownOptions
    {
        [AddInParameter("Web service URL"), AddInParameterEditor(typeof(TextParameterEditor), "inputClass=NewUIinput;TextArea=True;style=height:60px;")]
        public override string WebServiceURI { get; set; }

        [AddInParameter("Security key"), AddInParameterEditor(typeof(TextParameterEditor), "inputClass=NewUIinput;")]
        public override string SecurityKey { get; set; }

        [AddInParameter("Global tag for connection availability"), AddInParameterEditor(typeof(TextParameterEditor), "inputClass=NewUIinput;")]
        public string WebSeviceConnectionStatusGlobalTagName { get; set; }

        [AddInParameter("Shop"), AddInParameterEditor(typeof(DropDownParameterEditor), "NewGUI=true;none=false"), AddInParameterGroup("Source")]
        public string Shop { get; set; }

        [AddInParameter("Live integration for anonymous users"), AddInParameterEditor(typeof(YesNoParameterEditor), "NewUIcheckbox=true;")]
        public bool IsIntegrationForAnonymousUsersAllowed { get; set; }

        [AddInParameter("Disable Live Integration on Product Catalog"), AddInParameterEditor(typeof(YesNoParameterEditor), "NewUIcheckbox=true;")]
        public bool DisableLiveIntegrationOnProductCatalog { get; set; }

        [AddInParameter("Number format culture"), AddInParameterEditor(typeof(DropDownParameterEditor), "NewGUI=true;")]
        public string NumberFormatCulture { get; set; }

        [AddInParameter("Products information cache level"), AddInParameterEditor(typeof(DropDownParameterEditor), "NewGUI=true;none=false;")]
        public string ProductCacheLevel { get; set; }

        [AddInParameter("Orders cache level"), AddInParameterEditor(typeof(DropDownParameterEditor), "NewGUI=true;none=false;")]
        public string OrderCacheLevel { get; set; }

        [AddInParameter("Order state after export confirmed"), AddInParameterEditor(typeof(DropDownParameterEditor), "NewGUI=true;none=false;SortBy=Key;"), AddInParameterGroup("Source")]
        public virtual string OrderStateAfterExport { get; set; }

        [AddInParameter("Order state after export failed"), AddInParameterEditor(typeof(DropDownParameterEditor), "NewGUI=true;none=false;SortBy=Key;"), AddInParameterGroup("Source")]
        public virtual string OrderStateAfterExportFailed { get; set; }

        [AddInParameter("Text for discount orderlines:"),
         AddInParameterEditor(typeof(Dynamicweb.Extensibility.Editors.TextParameterEditor), "NewGUI=true;inputClass=NewUIinput;Horizontal=true;")]
        public string TextForDiscounts { get; set; }

        [AddInParameter("Notification recipient e-mail"), AddInParameterEditor(typeof(TextParameterEditor), "inputClass=NewUIinput;")]
        public string NotificationEmail { get; set; }
        [AddInParameter("Notification e-mail template"), AddInParameterEditor(typeof(TemplateParameterEditor), "folder=Templates/DataIntegration/Notifications;FullPath=true;inputClass=NewUIinput;")]
        public string NotificationTemplate { get; set; }
        [AddInParameter("Notification e-mail subject"), AddInParameterEditor(typeof(TextParameterEditor), "inputClass=NewUIinput;")]
        public string NotificationEmailSubject { get; set; }
        [AddInParameter("Notification e-mail sender name"), AddInParameterEditor(typeof(TextParameterEditor), "inputClass=NewUIinput;")]
        public string NotificationEmailSenderName { get; set; }
        [AddInParameter("Notification sender e-mail"), AddInParameterEditor(typeof(TextParameterEditor), "inputClass=NewUIinput;")]
        public string NotificationEmailSenderEmail { get; set; }

        [AddInParameter("Notification sending frequency"), AddInParameterEditor(typeof(DropDownParameterEditor), "NewGUI=true;none=false;SortBy=Key;")]
        public string NotificationSendingFrequency { get; set; }

        [AddInParameter("Log general errors"), AddInParameterEditor(typeof(YesNoParameterEditor), "NewUIcheckbox=true;")]
        public bool LogGeneralErrors { get; set; }

        [AddInParameter("Log connection errors"), AddInParameterEditor(typeof(YesNoParameterEditor), "NewUIcheckbox=true;")]
        public bool LogConnectionErrors { get; set; }

        [AddInParameter("Log response errors"), AddInParameterEditor(typeof(YesNoParameterEditor), "NewUIcheckbox=true;")]
        public bool LogResponseErrors { get; set; }

        [AddInParameter("Log request and response content"), AddInParameterEditor(typeof(YesNoParameterEditor), "NewUIcheckbox=true;")]
        public bool LogDebugInfo { get; set; }

        [AddInParameter("Log file max size (MB)"), AddInParameterEditor(typeof(TextParameterEditor), "inputClass=NewUIinput;")]
        public int LogMaxSize
        { get; set; }

        [AddInParameter("Include Order Custom Fields to request"), AddInParameterEditor(typeof(YesNoParameterEditor), "NewUIcheckbox=true;")]
        public bool AddOrderFieldsToRequest { get; set; }

        [AddInParameter("Include OrderLine Custom Fields to request"), AddInParameterEditor(typeof(YesNoParameterEditor), "NewUIcheckbox=true;")]
        public bool AddOrderLineFieldsToRequest { get; set; }

        [AddInParameter("Include Product Custom Fields to request"), AddInParameterEditor(typeof(YesNoParameterEditor), "NewUIcheckbox=true;")]
        public bool AddProductFieldsToRequest { get; set; }

        [AddInParameter("Allow Multiple Users per Company"), AddInParameterEditor(typeof(YesNoParameterEditor), "NewUIcheckbox=true;")]
        public bool AllowMultipleContactsPerCompany { get; set; }


        public LiveIntegrationAddIn()
        {
        }

        public override string LoadSettings()
        {
            WebServiceURI = Helpers.GetConfigValue(Constants.WebServiceConfiguration.Path, Constants.WebServiceConfiguration.Url);
            Shop = Helpers.GetConfigValue(Constants.WebServiceConfiguration.Path, Constants.WebServiceConfiguration.Shop);
            SecurityKey = Helpers.GetConfigValue(Constants.WebServiceConfiguration.Path, Constants.WebServiceConfiguration.SecurityKey);
            TextForDiscounts = Helpers.GetConfigValue(Constants.WebServiceConfiguration.Path,
                                                      Constants.WebServiceConfiguration.DiscountText);
            WebSeviceConnectionStatusGlobalTagName = Helpers.GetConfigValue(Constants.WebServiceConfiguration.Path, Constants.WebServiceConfiguration.WebSeviceConnectionStatusGlobalTagName);
            if (WebSeviceConnectionStatusGlobalTagName == null)
            {
                WebSeviceConnectionStatusGlobalTagName = "WebSeviceConnectionStatus";
            }

            IsIntegrationForAnonymousUsersAllowed = Convert.ToBoolean(Helpers.GetConfigValue(Constants.WebServiceConfiguration.Path, Constants.WebServiceConfiguration.AllowIntegrationForAnonymousUsers));
            DisableLiveIntegrationOnProductCatalog = Convert.ToBoolean(Helpers.GetConfigValue(Constants.WebServiceConfiguration.Path, Constants.WebServiceConfiguration.DisableLiveIntegrationOnProductCatalog));
            NumberFormatCulture = Helpers.GetConfigValue(Constants.WebServiceConfiguration.Path, Constants.WebServiceConfiguration.NumberFormatCulture);

            ProductCacheLevel = Helpers.GetConfigValue(Constants.CacheConfiguration.Path, Constants.CacheConfiguration.Product);
            OrderCacheLevel = Helpers.GetConfigValue(Constants.CacheConfiguration.Path, Constants.CacheConfiguration.Order);
            OrderStateAfterExport = Helpers.GetConfigValue(Constants.OrderSettings.Path, Constants.OrderSettings.OrderStateAfterExport);
            OrderStateAfterExportFailed = Helpers.GetConfigValue(Constants.OrderSettings.Path, Constants.OrderSettings.OrderStateAfterExportFailed);            

            LogConnectionErrors = Convert.ToBoolean(Helpers.GetConfigValue(Constants.ErrorHandlingConfiguration.Path, Constants.ErrorHandlingConfiguration.LogConnectionErrors));
            LogResponseErrors = Convert.ToBoolean(Helpers.GetConfigValue(Constants.ErrorHandlingConfiguration.Path, Constants.ErrorHandlingConfiguration.LogResponseErrors));
            LogGeneralErrors = Convert.ToBoolean(Helpers.GetConfigValue(Constants.ErrorHandlingConfiguration.Path, Constants.ErrorHandlingConfiguration.LogGeneralErrors));
            LogDebugInfo = Convert.ToBoolean(Helpers.GetConfigValue(Constants.ErrorHandlingConfiguration.Path, Constants.ErrorHandlingConfiguration.LogDebugInfo));
            LogMaxSize = Convert.ToInt32(Helpers.GetConfigValue(Constants.ErrorHandlingConfiguration.Path, Constants.ErrorHandlingConfiguration.LogMaxSize));

            if (LogMaxSize == 0)
                LogMaxSize = 10; // MB

            NotificationEmail = Helpers.GetConfigValue(Constants.NotificationSettings.Path, Constants.NotificationSettings.NotificationEmail);
            NotificationTemplate = Helpers.GetConfigValue(Constants.NotificationSettings.Path, Constants.NotificationSettings.NotificationTemplate);
            NotificationEmailSubject = Helpers.GetConfigValue(Constants.NotificationSettings.Path, Constants.NotificationSettings.NotificationEmailSubject);
            NotificationEmailSenderName = Helpers.GetConfigValue(Constants.NotificationSettings.Path, Constants.NotificationSettings.NotificationEmailSenderName);
            NotificationEmailSenderEmail = Helpers.GetConfigValue(Constants.NotificationSettings.Path, Constants.NotificationSettings.NotificationEmailSenderEmail);
            NotificationSendingFrequency = Helpers.GetConfigValue(Constants.NotificationSettings.Path, Constants.NotificationSettings.NotificationSendingFrequency);

            AddOrderFieldsToRequest = Convert.ToBoolean(Helpers.GetConfigValue(Constants.CustomFieldsSettings.Path, Constants.CustomFieldsSettings.AddOrderFieldsToRequest));
            AddOrderLineFieldsToRequest = Convert.ToBoolean(Helpers.GetConfigValue(Constants.CustomFieldsSettings.Path, Constants.CustomFieldsSettings.AddOrderLineFieldsToRequest));
            AddProductFieldsToRequest = Convert.ToBoolean(Helpers.GetConfigValue(Constants.CustomFieldsSettings.Path, Constants.CustomFieldsSettings.AddProductFieldsToRequest));
            AllowMultipleContactsPerCompany = Convert.ToBoolean(Helpers.GetConfigValue(Constants.CustomFieldsSettings.Path, Constants.CustomFieldsSettings.AllowMultipleContactsPerCompany));

            XDocument document = new XDocument(new XDeclaration("1.0", "utf-8", string.Empty));
            XElement root = new XElement("Parameters");
            document.Add(root);
            root.Add(CreateParameterNode(GetType(), "Web service URL", WebServiceURI));
            root.Add(CreateParameterNode(GetType(), "Shop", Shop));
            root.Add(CreateParameterNode(GetType(), "Security key", SecurityKey));
            root.Add(CreateParameterNode(GetType(), "Text for discount orderlines:", TextForDiscounts));
            root.Add(CreateParameterNode(GetType(), "Global tag for connection availability", WebSeviceConnectionStatusGlobalTagName));
            root.Add(CreateParameterNode(GetType(), "Live integration for anonymous users", IsIntegrationForAnonymousUsersAllowed.ToString()));
            root.Add(CreateParameterNode(GetType(), "Disable Live Integration on Product Catalog", DisableLiveIntegrationOnProductCatalog.ToString()));
            root.Add(CreateParameterNode(GetType(), "Number format culture", NumberFormatCulture));

            root.Add(CreateParameterNode(GetType(), "Products information cache level", ProductCacheLevel));
            root.Add(CreateParameterNode(GetType(), "Orders cache level", OrderCacheLevel));
            root.Add(CreateParameterNode(GetType(), "Order state after export confirmed", OrderStateAfterExport));
            root.Add(CreateParameterNode(GetType(), "Order state after export failed", OrderStateAfterExportFailed));

            root.Add(CreateParameterNode(GetType(), "Log general errors", LogGeneralErrors.ToString()));
            root.Add(CreateParameterNode(GetType(), "Log connection errors", LogConnectionErrors.ToString()));
            root.Add(CreateParameterNode(GetType(), "Log response errors", LogResponseErrors.ToString()));
            root.Add(CreateParameterNode(GetType(), "Log request and response content", LogDebugInfo.ToString()));
            root.Add(CreateParameterNode(GetType(), "Log file max size (MB)", LogMaxSize.ToString()));

            root.Add(CreateParameterNode(GetType(), "Notification recipient e-mail", NotificationEmail));
            root.Add(CreateParameterNode(GetType(), "Notification e-mail template", NotificationTemplate));
            root.Add(CreateParameterNode(GetType(), "Notification e-mail subject", NotificationEmailSubject));
            root.Add(CreateParameterNode(GetType(), "Notification e-mail sender name", NotificationEmailSenderName));
            root.Add(CreateParameterNode(GetType(), "Notification sender e-mail", NotificationEmailSenderEmail));
            root.Add(CreateParameterNode(GetType(), "Notification sending frequency", NotificationSendingFrequency));

            root.Add(CreateParameterNode(GetType(), "Include Order Custom Fields to request", AddOrderFieldsToRequest.ToString()));
            root.Add(CreateParameterNode(GetType(), "Include OrderLine Custom Fields to request", AddOrderLineFieldsToRequest.ToString()));
            root.Add(CreateParameterNode(GetType(), "Include Product Custom Fields to request", AddProductFieldsToRequest.ToString()));
            root.Add(CreateParameterNode(GetType(), "Allow Multiple Users per Company", AllowMultipleContactsPerCompany.ToString()));

            return document.ToString();
        }

        public override void SaveSettings()
        {
            Dictionary<string, string> values = new Dictionary<string, string>();
            values.Add(Constants.WebServiceConfiguration.Url, WebServiceURI);
            values.Add(Constants.WebServiceConfiguration.Shop, Shop);
            values.Add(Constants.WebServiceConfiguration.SecurityKey, SecurityKey);
            values.Add(Constants.WebServiceConfiguration.DiscountText, TextForDiscounts);
            values.Add(Constants.WebServiceConfiguration.WebSeviceConnectionStatusGlobalTagName, WebSeviceConnectionStatusGlobalTagName);
            values.Add(Constants.WebServiceConfiguration.AllowIntegrationForAnonymousUsers, IsIntegrationForAnonymousUsersAllowed.ToString());
            values.Add(Constants.WebServiceConfiguration.DisableLiveIntegrationOnProductCatalog, DisableLiveIntegrationOnProductCatalog.ToString());
            values.Add(Constants.WebServiceConfiguration.NumberFormatCulture, NumberFormatCulture);
            Configuration.ConfigurationHandler.SetConfig(Constants.WebServiceConfiguration.Path, values);

            values = new Dictionary<string, string>();
            values.Add(Constants.CacheConfiguration.Product, ProductCacheLevel);
            values.Add(Constants.CacheConfiguration.Order, OrderCacheLevel);
            Configuration.ConfigurationHandler.SetConfig(Constants.CacheConfiguration.Path, values);

            values = new Dictionary<string, string>();
            values.Add(Constants.ErrorHandlingConfiguration.LogGeneralErrors, LogGeneralErrors.ToString());
            values.Add(Constants.ErrorHandlingConfiguration.LogConnectionErrors, LogConnectionErrors.ToString());
            values.Add(Constants.ErrorHandlingConfiguration.LogResponseErrors, LogResponseErrors.ToString());
            values.Add(Constants.ErrorHandlingConfiguration.LogDebugInfo, LogDebugInfo.ToString());
            values.Add(Constants.ErrorHandlingConfiguration.LogMaxSize, LogMaxSize.ToString());
            Configuration.ConfigurationHandler.SetConfig(Constants.ErrorHandlingConfiguration.Path, values);

            values = new Dictionary<string, string>();
            values.Add(Constants.NotificationSettings.NotificationEmail, NotificationEmail);
            values.Add(Constants.NotificationSettings.NotificationTemplate, NotificationTemplate);
            values.Add(Constants.NotificationSettings.NotificationEmailSubject, NotificationEmailSubject);
            values.Add(Constants.NotificationSettings.NotificationEmailSenderName, NotificationEmailSenderName);
            values.Add(Constants.NotificationSettings.NotificationEmailSenderEmail, NotificationEmailSenderEmail);
            values.Add(Constants.NotificationSettings.NotificationSendingFrequency, NotificationSendingFrequency);
            Configuration.ConfigurationHandler.SetConfig(Constants.NotificationSettings.Path, values);

            values = new Dictionary<string, string>();
            values.Add(Constants.CustomFieldsSettings.AddOrderFieldsToRequest, AddOrderFieldsToRequest.ToString());
            values.Add(Constants.CustomFieldsSettings.AddOrderLineFieldsToRequest, AddOrderLineFieldsToRequest.ToString());
            values.Add(Constants.CustomFieldsSettings.AddProductFieldsToRequest, AddProductFieldsToRequest.ToString());
            values.Add(Constants.CustomFieldsSettings.AllowMultipleContactsPerCompany, AllowMultipleContactsPerCompany.ToString());
            Configuration.ConfigurationHandler.SetConfig(Constants.CustomFieldsSettings.Path, values);

            values = new Dictionary<string, string>();
            values.Add(Constants.OrderSettings.OrderStateAfterExport, OrderStateAfterExport);
            values.Add(Constants.OrderSettings.OrderStateAfterExportFailed, OrderStateAfterExportFailed);
            Configuration.ConfigurationHandler.SetConfig(Constants.OrderSettings.Path, values);

            Helpers.ClearCachedCultureInfo();
        }

        public override bool IsWebServiceConnectionAvailable()
        {
            bool succes = true;
            string error = string.Empty;
            string request = "<GetEcomData></GetEcomData>";//simple request for checking connection

            var WebServiceUrls = UrlHandler.Instance.GetWebServiceUrls();

            if (WebServiceUrls.Length == 0)
            {
                return false;
            }

            foreach (string url in WebServiceUrls)
            {
                string hostUrl = UrlHandler.Instance.GetUrl(url);
                try
                {
                    string response = ErpServiceCaller.GetDataFromRequestString(hostUrl, SecurityKey, request);
                }
                catch (Exception ex)
                {
                    succes = false;
                    error += string.Format("Url:{0} ConnectionError:{1}. ", hostUrl, ex.Message);
                    Logger.Instance.Log(ErrorLevel.ConnectionError, string.Format("Url: '{0}'. Error checking Web Service connection: '{1}'. Request: '{2}'.", hostUrl, ex.Message, request));
                }
            }

            if (!succes && !string.IsNullOrEmpty(error))
            {
                throw new Exception(error);
            }
            return succes;
        }

        public override void RetrievePDF(HttpRequest request, HttpResponse response)
        {
            string id = request["id"];
            string type = request["type"];
            bool forceDownload = Context.Current.Request.GetBoolean("forceDownload");

            string requestString = "<GetPDFForItem type=\"" + type + "\" id=\"" + id + "\"";
            User user = User.GetCurrentUser();
            if (user != null)
            {
                requestString += " externalUserID=\"" + Helpers.CustomerIdentifier + "\"";
            }
            requestString += "></GetPDFForItem>";

            string base64EncodedPDF = string.Empty;
            try
            {
                Logger.Instance.Log(ErrorLevel.DebugInfo, string.Format("Request RetrievePDF sent: '{0}'.", requestString));
                base64EncodedPDF = ErpServiceCaller.GetDataFromRequestString(UrlHandler.Instance.GetWebServiceUrl(), SecurityKey, requestString);
                Logger.Instance.Log(ErrorLevel.DebugInfo, string.Format("Response RetrievePDF received: '{0}'.", base64EncodedPDF));
            }
            catch (Exception ex)
            {
                Logger.Instance.Log(ErrorLevel.ResponseError, string.Format("Response RetrievePDF returned error: '{0}'.", ex.Message));
                throw ex;
            }

            MemoryStream inputStream = new MemoryStream();
            StreamWriter writer = new StreamWriter(inputStream);
            writer.Write(base64EncodedPDF);
            writer.Flush();
            inputStream.Position = 0;
            response.ContentType = "application/pdf";

            if (forceDownload)
            {
                string fileName = string.Format("IntegrationCustomerCenterItem{0}.pdf", id);
                string filePath = HttpContext.Current.Server.MapPath("/" + Dynamicweb.Content.Files.FilesAndFolders.GetFilesFolderName() + "/System/Log/LiveIntegration/" + fileName);
                using (Stream stream = File.OpenWrite(filePath))
                {
                    DecodeStream(inputStream, stream);
                }
                response.Clear();
                response.AddHeader("content-disposition", string.Format("attachment;filename={0}", fileName));
                response.WriteFile(filePath);
                response.Flush();
                File.Delete(filePath);
            }
            else
            {
                DecodeStream(inputStream, response.OutputStream);
                response.OutputStream.Flush();
                response.OutputStream.Close();
            }
        }

        public override List<string> GetSupportedCustomerCenterIntegrationCalls()
        {
            return new List<string>() { "OpenOrder", "Invoice", "Credit" };
        }

        public override Template RetrieveItemDetailsFromRemoteSystem(Template template, string callType, User user, string itemID)
        {
            return IntegrationCustomerCenterHandler.RetrieveItemDetailsFromRemoteSystem(template, callType, user, itemID);
        }

        public override Template RetrieveItemsListFromRemoteSystem(Template template, string callType, User user, int pageSize, int pageIndex, ref int totalItemsCount)
        {
            return IntegrationCustomerCenterHandler.RetrieveItemsListFromRemoteSystem(template, callType, user, pageSize, pageIndex, out totalItemsCount);
        }

        public static void DecodeStream(Stream inStream, Stream output)
        {
            System.Security.Cryptography.ICryptoTransform transform = new System.Security.Cryptography.FromBase64Transform();
            using (System.Security.Cryptography.CryptoStream cryptStream = new System.Security.Cryptography.CryptoStream(inStream, transform, System.Security.Cryptography.CryptoStreamMode.Read))
            {
                byte[] buffer = new byte[4096];
                int bytesRead = 0;
                bytesRead = cryptStream.Read(buffer, 0, buffer.Length);

                while (bytesRead > 0)
                {
                    output.Write(buffer, 0, bytesRead);
                    bytesRead = cryptStream.Read(buffer, 0, buffer.Length);

                }
            }
        }

        public Hashtable GetOptions(string name)
        {
            Hashtable options = new Hashtable();
            if (name == "Notification sending frequency")
            {
                foreach (NotificationFrequency frequencyLevel in Enum.GetValues(typeof(NotificationFrequency)))
                    options.Add(((int)frequencyLevel).ToString(), GetNotificationFrequencyText(frequencyLevel));
            }
            else if (name == "Order state after export confirmed" || name == "Order state after export failed")
            {
                options.Add("", "Leave unchanged");
                foreach (Orders.OrderState state in Orders.OrderState.GetAllOrderStates())
                {
                    options.Add(state.Id, state.Name);
                }
            }
            else if (name == "Shop")
            {
                options.Add("", "Any");
                var shops = Ecommerce.Shops.Shop.GetShops();
                foreach (Ecommerce.Shops.Shop ashop in shops)
                    options.Add(ashop.Id.ToString(), ashop.Name);
            }
            else if (name == "Number format culture")
            {                                                
                foreach (System.Globalization.RegionInfo region in Country.GetRegions())
                {                    
                    options.Add(region.TwoLetterISORegionName, region.TwoLetterISORegionName);
                }                
            }
            else
            {
                foreach (string cacheLevel in Enum.GetNames(typeof(CacheLevel)))
                    options.Add(cacheLevel, cacheLevel);
            }

            return options;
        }

        public string GetNotificationFrequencyText(NotificationFrequency frequency)
        {
            string result = string.Empty;
            switch (frequency)
            {
                case NotificationFrequency.Minute:
                    result = "1 minute";
                    break;
                case NotificationFrequency.FiveMinutes:
                    result = "5 minutes";
                    break;
                case NotificationFrequency.TenMinutes:
                    result = "10 minutes";
                    break;
                case NotificationFrequency.FifteenMinutes:
                    result = "15 minutes";
                    break;
                case NotificationFrequency.ThirtyMinutes:
                    result = "30 minutes";
                    break;
                case NotificationFrequency.OneHour:
                    result = "1 hour";
                    break;
                case NotificationFrequency.TwoHours:
                    result = "2 hours";
                    break;
                case NotificationFrequency.ThreeHours:
                    result = "3 hours";
                    break;
                case NotificationFrequency.SixHours:
                    result = "6 hours";
                    break;
                case NotificationFrequency.TwelveHours:
                    result = "12 hours";
                    break;
                case NotificationFrequency.OneDay:
                    result = "1 day";
                    break;
                default:
                    result = "Never";
                    break;
            }
            return result;
        }


    }
}
