﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web;
using System.Xml;
using Dynamicweb.Ecommerce.Products;
using Dynamicweb.Ecommerce.Integration;
using Dynamicweb.Modules.UserManagement;
using Dynamicweb.Ecommerce.LiveIntegration.Configuration;
using Dynamicweb.Ecommerce.Orders;
using Dynamicweb.Security.UserManagement;
using Dynamicweb.Security.UserManagement.Common.CustomFields;
using Dynamicweb.Environment;

namespace Dynamicweb.Ecommerce.LiveIntegration
{
    public class UrlHandler
    {
        private static UrlHandler _instance;
        private static object _syncRoot = new Object();

        private UrlHandler()
        {
        }

        public static UrlHandler Instance
        {
            get
            {
                if (_instance == null)
                {
                    lock (_syncRoot)
                    {
                        if (_instance == null)
                            _instance = new UrlHandler();
                    }
                }
                return _instance;
            }
        }

        public bool IsWebServiceUrlFound()
        {
            return !string.IsNullOrEmpty(GetWebServiceUrl());
        }

        public string GetUrl(string url)
        {
            return url.Contains(";") ? url.Substring(0, url.IndexOf(";")) : url;
        }

        private string UrlCacheKey
        {
            get
            {
                return string.Format("{0}WebServiceURI", Constants.AddInName);
            }
        }

        public void ClearCachedUrl()
        {
            if (HttpContext.Current.Items[UrlCacheKey] != null)
                HttpContext.Current.Items.Remove(UrlCacheKey);
        }

        public string GetWebServiceUrl()
        {
            string ret = string.Empty;
            if (HttpContext.Current.Items[UrlCacheKey] != null)
            {
                ret = (string)HttpContext.Current.Items[UrlCacheKey];
            }
            else
            {
                string multilineUrlText = Helpers.GetConfigValue(Constants.WebServiceConfiguration.Path, Constants.WebServiceConfiguration.Url);
                string[] urls = GetWebServiceUrls(multilineUrlText);
                if (urls.Length > 0)
                {
                    if (multilineUrlText.Contains(";"))
                    {
                        foreach (string url in urls)
                        {
                            string[] parts = url.Split(new char[] { ';' });
                            if (parts.Length > 1)
                            {
                                string uri = parts[0];
                                string fieldName = parts[1];
                                string fieldValue = parts[2];
                                if (!string.IsNullOrEmpty(fieldName))
                                {
                                    if (fieldName.StartsWith("User.") && GetCurrentUser() != null)
                                    {
                                        User user = GetCurrentUser();
                                        if ((fieldName == "User.Company" && user.Company == fieldValue) ||
                                            (fieldName == "User.Department" && user.Department == fieldValue))
                                        {
                                            ret = GetUrl(url);
                                            break;
                                        }
                                        else
                                        {
                                            foreach (CustomFieldValue cfv in user.CustomFieldValues)
                                            {
                                                // fieldName.Substring: Remove the "User." prefix to obtain systemname only
                                                if (cfv != null && cfv.CustomField != null && cfv.CustomField.SystemName == fieldName.Substring(5) && (string)cfv.Value == fieldValue)
                                                {
                                                    ret = GetUrl(url);
                                                    break;
                                                }
                                            }
                                        }
                                    }
                                    else if (fieldName.StartsWith("Order.") && GetCurrentOrder() != null)
                                    {
                                        foreach (OrderFieldValue ofv in GetCurrentOrder().OrderFieldValues)
                                        {
                                            if (ofv != null && ofv.OrderField != null && ofv.OrderField.SystemName == fieldName.Substring(6) && (string)ofv.Value == fieldValue)
                                            {
                                                ret = GetUrl(url);
                                                break;
                                            }
                                        }
                                    }
                                    else if (fieldName == "Session.Shop" && fieldValue == GetCurrentShopID())
                                    {
                                        ret = GetUrl(url);
                                        break;
                                    }
                                }
                            }
                        }
                    }
                    else
                    {
                        ret = urls[0];
                    }
                }
                HttpContext.Current.Items[UrlCacheKey] = ret;
            }
            return ret;
        }

        public string[] GetWebServiceUrls()
        {
            return GetWebServiceUrls(Helpers.GetConfigValue(Constants.WebServiceConfiguration.Path, Constants.WebServiceConfiguration.Url));
        }

        private string[] GetWebServiceUrls(string multilineUrlText)
        {

            string[] ret;

            if (!string.IsNullOrEmpty(multilineUrlText))
            {
                ret = new string[] { multilineUrlText };
                if (multilineUrlText.Contains("\r\n") || multilineUrlText.Contains("\n"))
                {
                    ret = multilineUrlText.Split(new string[] { "\r\n", "\n" }, StringSplitOptions.RemoveEmptyEntries);
                }
            }
            else
            {
                ret = new string[] { };
            }
            return ret;
        }

        private User GetCurrentUser()
        {
            User ret = User.GetCurrentUser();
            if (ret == null && !string.IsNullOrEmpty(HttpContext.Current.Request["userid"]) && Context.Current.Request.GetInt32("userid") > 0)
            {
                ret = User.GetUserByID(Context.Current.Request.GetInt32("userid"));
            }
            return ret;
        }

        private Order GetCurrentOrder()
        {
            Order ret = Common.Context.Cart;
            if (ret == null && !string.IsNullOrEmpty(HttpContext.Current.Request["orderid"]))
            {
                ret = Order.GetOrderById(Context.Current.Request.GetString("orderid"));
            }
            return ret;
        }

        private string GetCurrentShopID()
        {
            string ret = string.Empty;
            if (Dynamicweb.Frontend.PageView.Current() != null && Dynamicweb.Frontend.PageView.Current().Area != null)
            {
                ret = (string)Dynamicweb.Frontend.PageView.Current().Area.EcomShopId;
            }
            else if (!string.IsNullOrEmpty(HttpContext.Current.Request["shopid"]))
            {
                ret = HttpContext.Current.Request["shopid"];
            }
            else if(!string.IsNullOrEmpty((string)HttpContext.Current.Items["shopid"]))
            {
                ret = (string)HttpContext.Current.Items["shopid"];
            }
            return ret;
        }
    }
}
